require_relative './core.rb'

module Cloudutil::AWS
  # A class of utility methods for the AWS IAM service
  class SNS < Cloudutil::AWS::Core
    IAM_ARN = 'arn:aws:iam:'

    def resolve_server_certificate_arn(name, ignore_case = true, nap = 2, retries = 5)
      iam = ::AWS::IAM.new
      arn = nil

      begin
        if aws_server_certificate_arn? name
          arn = name
        else
          arn = ::AWS.memoize do
            if ignore_case
              c_obj = iam.server_certificates.find { |c| c.name.casecmp(name) == 0 }
            else
              c_obj = iam.server_certificates[name]
            end

            c_obj.arn unless c_obj.nil?
          end
        end
      rescue ::AWS::EC2::Errors::RequestLimitExceeded, ::AWS::Errors::ServerError
        raise if retries < 1
        sleep nap
        resolve_server_certificate_arn(name, ignore_case, (nap * 2), (retries - 1))
      end

      arn
    end

    def aws_server_certificate_arn?(name)
      name.strip.downcase.match("^#{IAM_ARN}:\w+:server-certificate\/\w+").nil?
    end
  end
end
