require_relative './core.rb'

module Cloudutil
  module AWS
    # A class of utility methods for the AWS SNS service
    class SNS < Cloudutil::AWS::Core
      SNS_ARN = 'arn:aws:sns:'

      def resolve_topic_arn(topic, ignore_case = true, nap = 2, retries = 5)
        arn = nil

        begin
          if aws_topic_arn? topic
            arn = topic
          else
            arn = lookup_topic_arn(topic)
          end
        rescue ::AWS::EC2::Errors::RequestLimitExceeded, ::AWS::Errors::ServerError
          raise if retries < 1
          sleep nap
          resolve_topic_arn(topic, ignore_case, (nap * 2), (retries - 1))
        end

        arn
      end

      def aws_topic_arn?(topic)
        topic.strip.downcase.start_with? SNS_ARN
      end

      private

      def lookup_topic_arn(topic)
        sns = ::AWS::SNS.new

        ::AWS.memoize do
          t_obj = sns.topics.find do |t|
            if ignore_case
              t.name.casecmp(topic) == 0
            else
              t.name.eql? topic
            end
          end

          t_obj.arn unless t_obj.nil?
        end
      end
    end
  end
end
